@@ -57,7 +57,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ipmiutil.h"
 
 static char *progname  = "ipmiutil";
-static char *progver   = "3.07";
+char *progver   = "3.12";
 // static char fdebug = 0;
 /*int ipmiutil(int argc, char **argv); */
 
@@ -136,8 +136,8 @@ main(int argc, char **argv)
    int i;
    char *psubcmd = "";
    
-   printf("%s ver %s\n", progname,progver); 
    if (argc < 2) {
+      printf("%s ver %s\n", progname,progver); 
       show_usage();
       ret = ERR_USAGE;
       goto do_exit;
@@ -168,7 +168,7 @@ main(int argc, char **argv)
    for (i = 0; i < NSUBCMDS; i++)
    {
        if (strcmp(argv[1],subcmds[i].tag) == 0) {
-	  psubcmd = argv[1];
+          psubcmd = argv[1];
           argc--;
           argv++;
           ret = subcmds[i].rtn(argc,argv);
