@@ -44,8 +44,13 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ievents.h"
 #include "isensor.h"
 
-static char * progver   = "3.07";
+#ifdef METACOMMAND
+extern char * progver;  /*from ipmiutil.c*/
+static char * progname  = "ipmiutil hpoem";
+#else
+static char * progver   = "3.08";
 static char * progname  = "ihpoem";
+#endif
 
 static char *redund_str(uchar b)
 {
